using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Windows;
using KellermanSoftware.NinjaDatabaseLite;
using NinjaDatabaseLite.Demo.ViewModels;

namespace NinjaDatabaseLite.Demo.DataAccess
{
    public static class DatabaseAccessLayer
    {
        private static NinjaDb connectedDatabase;
        public static NinjaDb ConnectedDatabase
        {
            get
            {

                if (connectedDatabase == null)
                {
                    NinjaDbConfig config = new NinjaDbConfig();
                    config.Storage = new IsolatedStorageDatabase("Bucket List");
                    connectedDatabase = new NinjaDb(config);
                }

                return connectedDatabase;
            }
        }

        private static ObservableCollection<GoalViewModel> items;
        public static ObservableCollection<GoalViewModel> Items
        {
            get
            {
                if (items == null)
                {
                    // Loading items from the db
                    List<GoalViewModel> itemsTable = LoadGoals();

                    // Loading it
                    items = new ObservableCollection<GoalViewModel>();
                    foreach (var itemViewModel in itemsTable)
                    {
                        itemViewModel.MarkOld();
                        items.Add(itemViewModel);
                    }
                }
                return items;
            }
        }

        public static void UpdateItem(GoalViewModel item)
        {
            if (item == null)
                return;

            try
            {
                List<GoalViewModel> table = LoadGoals();
                // Searching for item with the same id
                var searchedItem = table.Where(i => i.Id.Equals(item.Id)).FirstOrDefault();
                if (searchedItem == null)
                    return;
                // Updating : Removing the old object and replacing it with the new one.
                table.Remove(searchedItem);
                table.Add(item);
                SaveGoals(table);
            }
            catch (Exception)
            {
            }
        }

        public static void UpdateItems(IEnumerable<GoalViewModel> items)
        {
            if (items == null)
                return;
            List<GoalViewModel> table = LoadGoals();
            foreach (var item in items)
            {
                // Searching for item with the same id
                var searchedItem = table.Where(i => i.Id.Equals(item.Id)).FirstOrDefault();
                if (searchedItem == null)
                    return;
                // Updating : Removing the old object and replacing it with the new one.
                table.Remove(searchedItem);
                table.Add(item);
            }
            SaveGoals(table);
        }

        public static void DeleteItem(GoalViewModel item)
        {
            if (item == null)
                return;
            try
            {
                // Updating the items list
                if (Items.Contains(item))
                    Items.Remove(item);

                // Removing from DB
                List<GoalViewModel> table = LoadGoals();
                table.Remove(item);
                SaveGoals(table);
            }
            catch (Exception)
            {
            }
        }

        public static void DeleteItems(IEnumerable<GoalViewModel> items)
        {
            if (items == null)
                return;
            try
            {
                // Removing from DB
                List<GoalViewModel> table = LoadGoals();
                foreach (var item in items)
                {
                    table.Remove(item);
                    if (Items.Contains(item))
                        Items.Remove(item);
                }
                SaveGoals(table);
            }
            catch (Exception)
            {
            }
        }

        public static void AddItem(GoalViewModel item)
        {
            if (item == null)
                return;
            // Updating the list
            if (!Items.Contains(item))
                Items.Add(item);

            // Adding to the db
            List<GoalViewModel> table = LoadGoals();
            table.Add(item);
            SaveGoals(table);
        }

        public static void AddItems(IEnumerable<GoalViewModel> items)
        {
            if (items == null)
                return;
            // Updating the list
            // Adding to the db
            List<GoalViewModel> table = LoadGoals();
            foreach (var item in items)
            {
                table.Add(item);
                if (!Items.Contains(item))
                    Items.Add(item);
            }
            SaveGoals(table);
        }

        private static List<GoalViewModel> LoadGoals()
        {
            return ConnectedDatabase.Load<List<GoalViewModel>>("Goals") ?? new List<GoalViewModel>();
        }

        private static void SaveGoals(List<GoalViewModel> table)
        {
            ConnectedDatabase.Save("Goals", table);
        }
    }
}